==> glibc-2.2.4.tar.gz: glibc-2.2.4/sunrpc/pmap_prot2.c <==
/* @(#)pmap_prot2.c	2.1 88/07/29 4.0 RPCSRC */
/*
 * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify Sun RPC without charge, but are not authorized
 * to license or distribute it to anyone else except as part of a product or
 * program developed by the user.
 *
 * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 *
 * Sun RPC is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 *
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
 * OR ANY PART THEREOF.
 *
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even if
 * Sun has been advised of the possibility of such damages.
 *
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */
#if !defined(lint) && defined(SCCSIDS)
static char sccsid[] = "@(#)pmap_prot2.c 1.3 87/08/11 Copyr 1984 Sun Micro";
#endif

/*
 * pmap_prot2.c
 * Protocol for the local binder service, or pmap.
 *
 * Copyright (C) 1984, Sun Microsystems, Inc.
 */

#include <rpc/types.h>
#include <rpc/xdr.h>
#include <rpc/pmap_prot.h>


/*
 * What is going on with linked lists? (!)
 * First recall the link list declaration from pmap_prot.h:
 *
 * struct pmaplist {
 *      struct pmap pml_map;
 *      struct pmaplist *pml_map;
 * };
 *
 * Compare that declaration with a corresponding xdr declaration that
 * is (a) pointer-less, and (b) recursive:
 *
 * typedef union switch (bool_t) {
 *
 *      case TRUE: struct {
 *              struct pmap;
 *              pmaplist_t foo;
 *      };
 *
 *      case FALSE: struct {};
 * } pmaplist_t;
 *
 * Notice that the xdr declaration has no nxt pointer while
 * the C declaration has no bool_t variable.  The bool_t can be
 * interpreted as ``more data follows me''; if FALSE then nothing
 * follows this bool_t; if TRUE then the bool_t is followed by
 * an actual struct pmap, and then (recursively) by the
 * xdr union, pamplist_t.
 *
 * This could be implemented via the xdr_union primitive, though this
 * would cause a one recursive call per element in the list.  Rather than do
 * that we can ``unwind'' the recursion
 * into a while loop and do the union arms in-place.
 *
 * The head of the list is what the C programmer wishes to past around
 * the net, yet is the data that the pointer points to which is interesting;
 * this sounds like a job for xdr_reference!
 */
bool_t
xdr_pmaplist (xdrs, rp)
     XDR *xdrs;
     struct pmaplist **rp;
{
  /*
   * more_elements is pre-computed in case the direction is
   * XDR_ENCODE or XDR_FREE.  more_elements is overwritten by
   * xdr_bool when the direction is XDR_DECODE.
   */
  bool_t more_elements;
  int freeing = (xdrs->x_op == XDR_FREE);
  struct pmaplist **next = NULL;

  while (TRUE)
    {
      more_elements = (bool_t) (*rp != NULL);
      if (!xdr_bool (xdrs, &more_elements))
	return FALSE;
      if (!more_elements)
	return TRUE;		/* we are done */
      /*
       * the unfortunate side effect of non-recursion is that in
       * the case of freeing we must remember the next object
       * before we free the current object ...
       */
      if (freeing)
	next = &((*rp)->pml_next);
      if (!xdr_reference (xdrs, (caddr_t *) rp,
			  (u_int) sizeof (struct pmaplist),
			  (xdrproc_t) xdr_pmap))
	  return FALSE;
      rp = freeing ? next : &((*rp)->pml_next);
    }
}

==> glibc-2.3.tar.gz: glibc-2.3/sunrpc/pmap_prot2.c <==
/* @(#)pmap_prot2.c	2.1 88/07/29 4.0 RPCSRC */
/*
 * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify Sun RPC without charge, but are not authorized
 * to license or distribute it to anyone else except as part of a product or
 * program developed by the user.
 *
 * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 *
 * Sun RPC is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 *
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
 * OR ANY PART THEREOF.
 *
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even if
 * Sun has been advised of the possibility of such damages.
 *
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */
#if !defined(lint) && defined(SCCSIDS)
static char sccsid[] = "@(#)pmap_prot2.c 1.3 87/08/11 Copyr 1984 Sun Micro";
#endif

/*
 * pmap_prot2.c
 * Protocol for the local binder service, or pmap.
 *
 * Copyright (C) 1984, Sun Microsystems, Inc.
 */

#include <rpc/types.h>
#include <rpc/xdr.h>
#include <rpc/pmap_prot.h>


/*
 * What is going on with linked lists? (!)
 * First recall the link list declaration from pmap_prot.h:
 *
 * struct pmaplist {
 *      struct pmap pml_map;
 *      struct pmaplist *pml_map;
 * };
 *
 * Compare that declaration with a corresponding xdr declaration that
 * is (a) pointer-less, and (b) recursive:
 *
 * typedef union switch (bool_t) {
 *
 *      case TRUE: struct {
 *              struct pmap;
 *              pmaplist_t foo;
 *      };
 *
 *      case FALSE: struct {};
 * } pmaplist_t;
 *
 * Notice that the xdr declaration has no nxt pointer while
 * the C declaration has no bool_t variable.  The bool_t can be
 * interpreted as ``more data follows me''; if FALSE then nothing
 * follows this bool_t; if TRUE then the bool_t is followed by
 * an actual struct pmap, and then (recursively) by the
 * xdr union, pamplist_t.
 *
 * This could be implemented via the xdr_union primitive, though this
 * would cause a one recursive call per element in the list.  Rather than do
 * that we can ``unwind'' the recursion
 * into a while loop and do the union arms in-place.
 *
 * The head of the list is what the C programmer wishes to past around
 * the net, yet is the data that the pointer points to which is interesting;
 * this sounds like a job for xdr_reference!
 */
bool_t
xdr_pmaplist (xdrs, rp)
     XDR *xdrs;
     struct pmaplist **rp;
{
  /*
   * more_elements is pre-computed in case the direction is
   * XDR_ENCODE or XDR_FREE.  more_elements is overwritten by
   * xdr_bool when the direction is XDR_DECODE.
   */
  bool_t more_elements;
  int freeing = (xdrs->x_op == XDR_FREE);
  struct pmaplist **next = NULL;

  while (TRUE)
    {
      more_elements = (bool_t) (*rp != NULL);
      if (!INTUSE(xdr_bool) (xdrs, &more_elements))
	return FALSE;
      if (!more_elements)
	return TRUE;		/* we are done */
      /*
       * the unfortunate side effect of non-recursion is that in
       * the case of freeing we must remember the next object
       * before we free the current object ...
       */
      if (freeing)
	next = &((*rp)->pml_next);
      if (!INTUSE(xdr_reference) (xdrs, (caddr_t *) rp,
				  (u_int) sizeof (struct pmaplist),
				  (xdrproc_t) INTUSE(xdr_pmap)))
	  return FALSE;
      rp = freeing ? next : &((*rp)->pml_next);
    }
}
INTDEF(xdr_pmaplist)
